<?php
declare(strict_types=1);

namespace Catequil\Invoice\Hydrator;

use Catequil\Invoice\Model\InvoiceAddress;
use Catequil\Invoice\Model\InvoiceInterface;
use Catequil\Invoice\Model\InvoiceItem;
use Catequil\Invoice\Model\InvoiceOrder;
use Catequil\Invoice\Model\InvoiceOrderInterface;
use Catequil\Invoice\Model\InvoicePayment;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Laminas\Hydrator\HydratorInterface;
use Laminas\Hydrator\NamingStrategy\MapNamingStrategy;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\Hydrator\Strategy\ClosureStrategy;
use Laminas\Hydrator\Strategy\CollectionStrategy;
use Laminas\Hydrator\Strategy\DateTimeFormatterStrategy;
use Laminas\Hydrator\Strategy\HydratorStrategy;
use Laminas\Hydrator\Strategy\StrategyChain;
use function array_map;
use function preg_match;

/**
 * Class InvoiceHydrator
 *
 * @package Catequil\Invoice\Hydrator
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-invoice
 */
final class InvoiceHydrator extends ReflectionHydrator
{
    private HydratorInterface $hydrator;

    public function __construct(HydratorInterface $hydrator)
    {
        $this->hydrator = $hydrator;
        $this->setNamingStrategy(
            MapNamingStrategy::createFromAsymmetricMap([], [
                'Id' => 'invoiceNo',
                'Datum' => 'createdAt',
                'WooIdNarudzbe' => 'orders',
                'WebShopSiteName' => 'webshop',
                'Komentar' => 'comment',
                'ProdajaNetto' => 'totalWoTax',
                'ProdajaBruto' => 'total',
                'PDV' => 'tax',
                'NacinDostave' => 'shipping',
                'OznValute' => 'currency',
                'Naplata' => 'billingAddress',
                'Dostava' => 'shippingAddress',
                'Body' => 'items',
                'NacinPlacanja' => 'payments',
                'PDFDoc' => 'pdf',
            ])
        );

        $this->addStrategy('billingAddress', new HydratorStrategy($hydrator, InvoiceAddress::class));
        $this->addStrategy('createdAt', new DateTimeFormatterStrategy('d.m.Y G:i:s'));
        $this->addStrategy('orders', new StrategyChain([
            new ClosureStrategy(
                fn (Collection $orders) => $orders->toArray(),
                fn (array $orders, $context) => new ArrayCollection($orders),
            ),
            new CollectionStrategy(
                $hydrator,
                InvoiceOrder::class
            ),
            new ClosureStrategy(
                fn (array $orders) => array_map(
                    fn (InvoiceOrderInterface $order) => 'Woo-' . $order->getOrderNo(),
                    $orders
                ),
                fn (array $orders, $context) => array_map(
                    function (string $order) use ($context) {
                        $matches = [];
                        preg_match('/(\d+)/', $order, $matches);
                        return ['orderNo' => $matches[1] ?? 0];
                    },
                    $orders
                ),
            )
        ]));
        $this->addStrategy('items', new StrategyChain([
            new ClosureStrategy(
                fn (Collection $items) => $items->toArray(),
                fn (array $items) => new ArrayCollection($items)
            ),
            new CollectionStrategy(
                $hydrator,
                InvoiceItem::class
            )
        ]));
        $this->addStrategy('payments', new StrategyChain([
            new ClosureStrategy(
                fn (Collection $payments) => $payments->toArray(),
                fn (array $payments) => new ArrayCollection($payments)
            ),
            new CollectionStrategy(
                $hydrator,
                InvoicePayment::class
            ),
        ]));
        $this->addStrategy('shippingAddress', new HydratorStrategy($hydrator, InvoiceAddress::class));
    }

    public function hydrate(array $data, object $object): InvoiceInterface
    {
        if (isset($data['Id']) && preg_match('/^([A-Z]+)-(\d+)$/', $data['Id'], $matches)) {
            $data['Id'] = $matches[2];
            $data['documentType'] = $matches[1];
        }

        /** @var InvoiceInterface $invoice */
        $invoice = parent::hydrate($data, $object);

        foreach ($invoice->getOrders() as $order) {
            $this->hydrator->hydrate(['invoice' => $invoice], $order);
        }

        foreach ($invoice->getPayments() as $payment) {
            $this->hydrator->hydrate(['invoice' => $invoice], $payment);
        }

        foreach ($invoice->getItems() as $item) {
            $this->hydrator->hydrate(['invoice' => $invoice], $item);
        }

        return $invoice;
    }
}